//---------------------------------------------------------------------------

#include <windows.h>
#include <math.h>
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------

#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
    return 1;
}
//---------------------------------------------------------------------------


extern "C" __declspec(dllexport) char init(void);
extern "C" __declspec(dllexport) void eval(void);

extern "C" __declspec(dllexport) float Voltage;
extern "C" __declspec(dllexport) float Current;
extern "C" __declspec(dllexport) float Resistance;
extern "C" __declspec(dllexport) float EMFVoltage;
extern "C" __declspec(dllexport) float ConstantV;
extern "C" __declspec(dllexport) float Inertia;
extern "C" __declspec(dllexport) float Friction;
extern "C" __declspec(dllexport) float Torque;
extern "C" __declspec(dllexport) float Epsilon;
extern "C" __declspec(dllexport) float Omega;
extern "C" __declspec(dllexport) float Speed;
extern "C" __declspec(dllexport) float Position;
extern "C" __declspec(dllexport) float DeltaT;



float Voltage=0;
float Current=0;
float Resistance=2.5;
float EMFVoltage=0;
float ConstantV=0.3;
float Inertia=0.01;
float Friction=0.001;
float Torque=0;
float Epsilon=0;
float Omega=0;
float Speed=0;
float Position=0;
float PositionOld=0;
float DeltaT=0.005;


int DLLHooks=0;


char init(void)
{

    PositionOld = Position;

    DLLHooks++;
    return DLLHooks;

}


void eval(void)
{

    if(Resistance < 1e-3)
        Resistance = 1e-3;

    if(Inertia < 1e-6)
        Inertia = 1e-6;

    Speed = (Speed + (Position - PositionOld) / DeltaT) / 2;
    PositionOld = Position;

    if(fabs(Speed) > 0.1)
        Omega = Speed * 2 * M_PI;

    Current = (Voltage - EMFVoltage) / Resistance;
    Torque = ConstantV * Current;
    Epsilon = (Torque - Friction * Omega) / Inertia;
    Omega += Epsilon * DeltaT;

    EMFVoltage = ConstantV * Omega;

    Speed = Omega / (2 * M_PI);
    Position += Speed * DeltaT;

}
